import requests
import json

# Some variables for not repeating the same thing

INFO_ACCEPT_HEADER = {
    'Accept': 'application/vnd.ga4gh.refget.v1.0.0+json'
}
INFO_API = 'sequence/service-info'


def info_implement(test, runner):
    '''
    Test to check if info-endpoint returns 200 OK with appropriate headers
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + INFO_API, headers=INFO_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def info_implement_default(test, runner):
    '''
    Test to check if info-endpoint returns 200 OK without headers
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + INFO_API)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def info_circular(test, runner):
    '''
    Test to check if info-endpoint has circular in the response object. And if
    it is there it updates session_params['circular'] as per the value
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    response = requests.get(base_url + INFO_API, headers=INFO_ACCEPT_HEADER)
    service_info_object = None
    try:
        service_info_object = json.loads(response.text)["service"]
        test.result = 1
        if service_info_object['circular_supported'] == True:
            session_params['circular_supported'] = True
        else:
            session_params['circular_supported'] = False
    except:
        test.result = -1
        test.fail_text = test.fail_text + str(service_info_object)


def info_algorithms(test, runner):
    '''
    Test to check if info-endpoint has algorithms in the response object. And if
    it is there it updates session_params['trunc512'] as per the value
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    response = requests.get(base_url + INFO_API, headers=INFO_ACCEPT_HEADER)
    service_info_object = None
    try:
        service_info_object = json.loads(response.text)["service"]
        test.result = 1
        if 'trunc512' in service_info_object['algorithms']:
            session_params['trunc512'] = True
        else:
            session_params['trunc512'] = False
    except:
        test.result = -1
        test.fail_text = test.fail_text + str(service_info_object)


def info_subsequence(test, runner):
    '''
    Test to check if info-endpoint has subsequence_limit in the response object.
    If the key is present we update session_params['subsequence_limit'] as per the
    value
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    service_info_object = None
    response = requests.get(base_url + INFO_API, headers=INFO_ACCEPT_HEADER)
    try:
        service_info_object = json.loads(response.text)["service"]
        if service_info_object['subsequence_limit'] is not None:
            session_params['subsequence_limit'] = int(service_info_object['subsequence_limit'])
        test.result = 1
    except:
        test.result = -1
        test.fail_text = test.fail_text + str(service_info_object)


def info_api_version(test, runner):
    '''
    Test to check if info-endpoint has supported_api_versions in the response
    object.
    '''
    service_info_object = None
    base_url = str(runner.base_url)
    response = requests.get(base_url + INFO_API, headers=INFO_ACCEPT_HEADER)
    try:
        service_info_object = json.loads(response.text)["service"]
        if "supported_api_versions" in service_info_object:
            test.result = 1
    except:
        test.result = -1
        test.fail_text = test.fail_text + str(service_info_object)



import os

TEST_SUITE = os.path.dirname(__file__)
SEQUENCE_FILES = os.path.join(TEST_SUITE, 'sequences/')


class Sequence:
    '''Sequence model object used to store sequence data in a defined manner
    '''
    def __init__(self, name, sequence, is_circular, sha512, md5, size):
        self.name = name
        self.sequence = sequence
        self.is_circular = is_circular
        self.sha512 = sha512
        self.md5 = md5
        self.size = size


# Additional utility functions to load the sequence data


def read_sequence(chr_name):
    with open(SEQUENCE_FILES + chr_name + ".faa", "r") as sequence_file:
        next(sequence_file)
        sequence_data = sequence_file.read().replace('\n', '')
    return sequence_data


def read_sequence_data(chr_name):
    import json
    with open(SEQUENCE_FILES + "checksums.json", "r") as checksums_file:
        checksums = json.load(checksums_file)
    return checksums[chr_name]


def get_seq_obj(chr):
    data = read_sequence_data(chr)
    if data['is_circular'] == 1:
        data['is_circular'] = True
    else:
        data['is_circular'] = False
    return Sequence(
        chr,
        read_sequence(chr),
        data['is_circular'],
        data['sha512'],
        data['md5'],
        len(read_sequence(chr))
    )


def data():
    '''data fixture loads all the data and return data variable for use in tests
    '''
    data = []
    data.append(get_seq_obj("I"))
    data.append(get_seq_obj("VI"))
    data.append(get_seq_obj("NC"))
    return data



from compliance_suite.tests import initiate_tests
from compliance_suite.utils import data
import datetime
import re
import sys


def processed_func_descrp(text):
    return re.sub(' +', ' ', text.replace('\n', '')).strip()


class TestRunner():
    '''
    Test runner class used to run tests, pass context variables through test
    graph to generate report appropriately.
    '''
    def __init__(self, base_url):
        '''
        Required params:
            base_url - The base url of the server on which the report will be
                generated

        Object params:
            session_params - Variables from info endpoint used to run conditional
                tests. Values are populated when info endpoint tests are run
            total_tests - Total number of tests
            total_tests_passed - Total number of tests passed
            total_tests_failed - Total number of tests failed
            total_tests_skipped - Total number of tests skipped
            total_tests_warning - Total number of tests generated warning
            results - To store test result objects for report generation
        '''

        self.root = None
        self.session_params = {
            'limit': None,
            'trunc512': None,
            'circular': None,
            'redirection': None
        }
        self.test_data = data()
        self.total_tests = 0
        self.total_tests_passed = 0
        self.total_tests_skipped = 0
        self.total_tests_failed = 0
        self.total_tests_warning = 0
        self.base_url = base_url
        self.results = []

    def recurse_label_tests(self, root):
        '''
        Labels the test case nodes and populate label parameter in test objects.
        to ensure parents are run first no matter the shape of test graph
        '''
        label = root.label + 1
        for child in root.children:
            if label > child.label:
                self.total_tests = self.total_tests + 1
                child.label = label
            if len(child.children) != 0:
                self.recurse_label_tests(child)

    def recurse_generate_json(self, node):
        '''
        Generate the report according to the label set in above function by
        test_result_object s. It also populates other params of test_runner
        object
        '''
        label = node.label + 1
        for child in node.children:
            if child.label == label:
                test_result_object = {
                    'name': str(child),
                    'result': child.result,
                    'test_description': processed_func_descrp(child.algorithm.__doc__),
                    'text': child.to_echo(),
                    'parents': [str(parent) for parent in child.parents],
                    'children': [str(child) for child in child.children],
                    'warning': child.warning,
                    'edge_cases': child.case_outputs,
                    # 'edge_cases': [case for case in child.case_outputs if case['result'] == -1]
                }
                if child.result == 1:
                    self.total_tests_passed = self.total_tests_passed + 1
                elif child.result == -1:
                    self.total_tests_failed = self.total_tests_failed + 1
                else:
                    self.total_tests_skipped = self.total_tests_skipped + 1
                if child.warning is True:
                    self.total_tests_warning = self.total_tests_warning + 1
                self.results.append(test_result_object)
                if len(child.children) != 0:
                    self.recurse_generate_json(child)

    def recurse_run_tests(self, node):
        '''
        Runs the test graph according to the label set
        '''
        label = node.label + 1
        for child in node.children:
            if child.label == label:
                print(str(child), file=sys.stderr)
                child.run(self)
        for child in node.children:
            if len(child.children) != 0:
                self.recurse_run_tests(child)

    def generate_final_json(self):
        '''
        Generate final report object for this session
        '''
        now = datetime.datetime.now()
        report_object = {
            'server': self.base_url,
            'date_time': str(now),
            'test_results': self.results,
            'total_tests': self.total_tests,
            'total_tests_passed': self.total_tests_passed,
            'total_tests_skipped': self.total_tests_skipped,
            'total_tests_failed': self.total_tests_failed,
            'total_warnings': self.total_tests_warning
        }
        return report_object

    def run_tests(self):
        '''
        The controller function of the test runner object. Defines the
        complete pipeline.
        '''
        self.root = initiate_tests()
        self.root.run(self)
        self.recurse_label_tests(self.root)
        self.recurse_run_tests(self.root)
        self.recurse_generate_json(self.root)


if __name__ == "__main__":
    tr = TestRunner('https://www.ebi.ac.uk/ena/cram/')
    tr.run_tests()

    print('--------------', file=sys.stderr)

    # tr = TestRunner('http://localhost:5000/')
    # tr.run_tests()



import requests


SEQUENCE_ACCEPT_HEADER = {
    'Accept': 'text/vnd.ga4gh.refget.v1.0.0+plain'
}
SEQUENCE_MD5 = 'sequence/6681ac2f62509cfc220d78751b8dc524'
SEQUENCE_TRUNC512 = 'sequence/959cb1883fc1ca9ae1394ceb475a356ead1ecceff5824ae7'
SEQUENCE_CIRCULAR = 'sequence/3332ed720ac7eaa9b3655c06f6b9e196'


def sequence_implement(test, runner):
    '''Test to check if server returns 200 using I test sequence and
    appropriate headers
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + SEQUENCE_MD5, headers=SEQUENCE_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def sequence_implement_default(test, runner):
    '''Test to check if server returns 200 using I test sequence and
    no headers
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + SEQUENCE_MD5, headers=SEQUENCE_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def sequence_query_by_trunc512(test, runner):
    '''Test to check if server returns 200 using I test sequence trunc512 and
    appropriate headers if the server supports trunc512
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    if session_params['trunc512'] is False:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because server does not support TRUNC512 algorithm')
        return
    response = requests.get(base_url + SEQUENCE_MD5, headers=SEQUENCE_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def sequence_invalid_checksum_404_error(test, runner):
    '''Test to check if server returns 404 using some garbage checksum and
    appropriate headers
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + 'sequence/Garbagechecksum', headers=SEQUENCE_ACCEPT_HEADER)
    if response.status_code == 404:
        test.result = 1
    else:
        test.result = -1
        test.fail_text = test.fail_text + str(response.status_code)


def sequence_invalid_encoding_406_error(test, runner):
    '''Test to check if server returns 200 using I test sequence and
    garbage encoding in Accept header
    '''
    base_url = str(runner.base_url)
    response = requests.get(
        base_url + SEQUENCE_MD5,
        headers={'Accept': 'embl/some_json'})
    if response.status_code == 406:
        test.result = 1
    else:
        test.result = -1
        test.fail_text = test.fail_text + str(response.status_code)


def sequence_start_end(test, runner):
    '''Test to check if server returns 200 and appropriate text using I test
    sequence and start/end query params set to 10 and 20 respectively
    '''
    base_url = str(runner.base_url)
    response = requests.get(
        base_url + SEQUENCE_MD5 + '?start=10&end=20',
        headers=SEQUENCE_ACCEPT_HEADER)
    if response.status_code == 200 and response.text == 'CCCACACACC':
        test.result = 1
    else:
        test.result = -1


def sequence_start_end_success_cases(test, runner):
    '''Test to check if server passes all the edge cases related to success
    queries using start/end params
    '''
    data = runner.test_data
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        response = requests.get(
            base_url + SEQUENCE_MD5 + _input[0],
            headers=SEQUENCE_ACCEPT_HEADER)
        case_output_object = {'api': SEQUENCE_MD5 + ':' + _input[0] + ':' + str(SEQUENCE_ACCEPT_HEADER)}
        if response.status_code == 200 and \
                response.text == data[0].sequence[_input[1]:_input[2]] and \
                int(response.headers['content-length']) == _output:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)


def sequence_range(test, runner):
    '''Test to check if server returns 200 and appropriate text using I test
    sequence and range header set to 10 and 19 respectively
    '''
    base_url = str(runner.base_url)
    header = {
        'Accept': 'text/vnd.ga4gh.refget.v1.0.0+plain',
        'Range': 'bytes=10-19'
    }
    response = requests.get(
        base_url + SEQUENCE_MD5, headers=header)
    if response.status_code == 206 and response.text == 'CCCACACACC':
        test.result = 1
    else:
        test.result = -1


def sequence_range_success_cases(test, runner):
    '''Test to check if server passes all the edge cases related to range header
    success queries
    '''
    data = runner.test_data
    header = {
        'Accept': 'text/vnd.ga4gh.refget.v1.0.0+plain',
    }
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        header['Range'] = _input[0]
        response = requests.get(
            base_url + SEQUENCE_MD5, headers=header)
        case_output_object = {'api': SEQUENCE_MD5 + ':' + _input[0] + ':' + str(SEQUENCE_ACCEPT_HEADER)}
        if response.status_code == _output[0] and \
                response.text == data[0].sequence[_input[1]:_input[2] + 1] \
                and int(response.headers['content-length']) == _output[1]:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)


def sequence_circular(test, runner):
    '''Test to check if server passes all the edge cases related to circular queries
    '''
    session_params = runner.session_params
    if session_params['circular_supported'] is False:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because server does not support circular sequences')
        return
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        response = requests.get(
            base_url + SEQUENCE_CIRCULAR + _input, headers=SEQUENCE_ACCEPT_HEADER)
        case_output_object = {'api': SEQUENCE_CIRCULAR + _input + ':' + str(SEQUENCE_ACCEPT_HEADER)}
        if response.status_code == 200 and \
                response.text == _output[0] \
                and int(response.headers['content-length']) == _output[1]:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)


def sequence_start_end_errors(test, runner):
    '''Test to check if server passes all the edge cases related start-end
    error cases
    '''
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        response = requests.get(
            base_url + 'sequence/' + _input[0] + _input[1], headers=SEQUENCE_ACCEPT_HEADER)
        case_output_object = {'api': 'sequence/' + _input[0] + ':' + _input[1]}
        if response.status_code == _output:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)


def sequence_range_errors(test, runner):
    '''Test to check if server passes all the edge cases related range
    error cases
    '''
    header = {
        'Accept': 'text/vnd.ga4gh.refget.v1.0.0+plain',
    }
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        header['Range'] = _input[1]
        response = requests.get(
            base_url + 'sequence/' + _input[0], headers=header)
        case_output_object = {'api': _input[0] + ':' + _input[1]}
        if response.status_code == _output:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)


def sequence_circular_support_false_errors(test, runner):
    '''Test to check if server throws correct error codes on circular sequence
    query if server does not support circular sequences
    '''
    session_params = runner.session_params
    if session_params['circular_supported'] is True:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because server supports circular sequences')
        return
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        response = requests.get(
            base_url + 'sequence/' + _input[0] + _input[1], headers=SEQUENCE_ACCEPT_HEADER)
        case_output_object = {'api': 'sequence/' + _input[0] + ':' + _input[1]}
        if response.status_code == _output:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)


def sequence_circular_support_true_errors(test, runner):
    '''Test to check if server throws correct error codes on circular sequence
    query if server supports circular sequences
    '''
    session_params = runner.session_params
    if session_params['circular_supported'] is False:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because server does not support circular sequences')
        return
    base_url = str(runner.base_url)
    test.result = 1
    for case in test.cases:
        _input = case[0]
        _output = case[1]
        response = requests.get(
            base_url + 'sequence/' + _input[0] + _input[1], headers=SEQUENCE_ACCEPT_HEADER)
        case_output_object = {'api': 'sequence/' + _input[0] + ':' + _input[1]}
        if response.status_code == _output:
            case_output_object['result'] = 1
        else:
            case_output_object['result'] = -1
            test.result = -1
        test.case_outputs.append(case_output_object)



import requests
import json

# Some variables for not repeating the same thing

METADATA_ACCEPT_HEADER = {
    'Accept': 'application/vnd.ga4gh.refget.v1.0.0+json'
}
SEQ_MD5 = '6681ac2f62509cfc220d78751b8dc524/metadata'
SEQ_TRUNC512 = '959cb1883fc1ca9ae1394ceb475a356ead1ecceff5824ae7/metadata'
SEQ_CIRCULAR = '3332ed720ac7eaa9b3655c06f6b9e196/metadata'


def metadata_implement(test, runner):
    '''
    Test if metadata endpoint returns 200 with appropriate headers using I test
    sequence
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + 'sequence/' + SEQ_MD5, headers=METADATA_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def metadata_implement_default(test, runner):
    '''
    Test if metadata endpoint returns 200 without headers using I test sequence
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + 'sequence/' + SEQ_MD5)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def metadata_query_by_trunc512(test, runner):
    '''
    Test if metadata endpoint returns 200 using trunc512 with I test sequence
    if the server supports. Value stored in session_params. If not skip test
    and set skip text appropriately
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    if session_params['trunc512'] is False:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because server does not support TRUNC512 algorithm')
        return
    response = requests.get(base_url + 'sequence/' + SEQ_TRUNC512, headers=METADATA_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def metadata_query_circular_sequence(test, runner):
    '''
    Test if metadata endpoint returns 200 using circular test sequence
    if the server supports circular sequences. Value stored in session_params.
    If not skip test and set skip text appropriately
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    if session_params['circular_supported'] is False:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because server does not support circular sequences')
        return
    response = requests.get(base_url + 'sequence/' + SEQ_CIRCULAR, headers=METADATA_ACCEPT_HEADER)
    if response.status_code == 200:
        test.result = 1
    else:
        test.result = -1


def metadata_md5(test, runner):
    '''
    Test if md5 in metadata response object
    '''
    base_url = str(runner.base_url)
    test.result = -1
    response = requests.get(base_url + 'sequence/' + SEQ_MD5, headers=METADATA_ACCEPT_HEADER)
    try:
        metadata_object = json.loads(response.text)["metadata"]
        if "md5" in metadata_object:
            test.result = 1
    except:
        test.fail_text = test.fail_text + str(metadata_object)


def metadata_trunc512(test, runner):
    '''
    Test if trunc512 in metadata response object. Skip if server does not
    support trunc512
    '''
    base_url = str(runner.base_url)
    session_params = runner.session_params
    test.result = -1
    if session_params['trunc512'] is False:
        test.result = 0
        test.set_skip_text(str(test) + ' is skipped because key "trunc512" although it is supported by the server')
        return
    try:
        response = requests.get(base_url + 'sequence/' + SEQ_MD5, headers=METADATA_ACCEPT_HEADER)
        metadata_object = json.loads(response.text)["metadata"]
        if "trunc512" in metadata_object:
            test.result = 1
    except:
        test.fail_text = test.fail_text + str(metadata_object)


def metadata_length(test, runner):
    '''
    Test if length in metadata response object
    '''
    base_url = str(runner.base_url)
    test.result = -1
    try:
        response = requests.get(base_url + 'sequence/' + SEQ_MD5, headers=METADATA_ACCEPT_HEADER)
        metadata_object = json.loads(response.text)["metadata"]
        if "length" in metadata_object and metadata_object['length'] == 230218:
            test.result = 1
    except:
        test.fail_text = test.fail_text + str(metadata_object)


def metadata_aliases(test, runner):
    '''
    Test if aliases in metadata response object
    '''
    base_url = str(runner.base_url)
    test.result = -1
    try:
        response = requests.get(base_url + 'sequence/' + SEQ_MD5, headers=METADATA_ACCEPT_HEADER)
        metadata_object = json.loads(response.text)["metadata"]
        if "aliases" in metadata_object:
            test.result = 1
    except:
        test.fail_text = test.fail_text + str(metadata_object)


def metadata_invalid_checksum_404_error(test, runner):
    '''
    Test if 404 on invalid checksum in metadata response
    '''
    base_url = str(runner.base_url)
    response = requests.get(base_url + 'sequence/' + 'some1111garbage1111ID', headers=METADATA_ACCEPT_HEADER)
    if response.status_code == 404:
        test.result = 1
    else:
        test.result = -1
        test.fail_text = test.fail_text + str(response.status_code)


def metadata_invalid_encoding_406_error(test, runner):
    '''
    Test if 406 on invalid encoding in Accept header
    '''
    base_url = str(runner.base_url)
    response = requests.get(
        base_url + 'sequence/' + SEQ_MD5,
        headers={'Accept': 'embl/some_json'})
    if response.status_code == 406:
        test.result = 1
    else:
        test.result = -1
        test.fail_text = test.fail_text + str(response.status_code)



from compliance_suite.info_algorithms import *
from compliance_suite.metadata_algorithms import *
from compliance_suite.sequence_algorithms import *
import sys

class Test():
    '''
    Test Case class. All the test cases are instances of this class.

    label - used in graph algorithms to label the test case graph
    algorithm - Strategy design pattern in used here. It is the underlying algorithm used in the test case
    result - 0 indicates skipped, 1 indicates passed, -1 indicates failed and 2 is not yet ran
    pass_text - text in the report when test case is passed
    skip_text - text in the report when test case is skipped
    fail_text - text in the report when test case is failed
    parents - dependencies of the test case to run
    children - test cases which have this test case as dependency
    warning - if the result of this test case is warning for the server implementation
    cases - multiple edge cases of same test object
    '''
    def __init__(self, algorithm):
        '''
        Initiates the Test Case object. Algorithm is a required field to initiate test case object
        '''
        self.label = 0
        self.algorithm = algorithm
        self.result = 2
        self.pass_text = ''
        self.fail_text = ''
        self.skip_text = ''
        self.parents = []
        self.children = []
        self.warning = False
        self.cases = []
        self.case_outputs = []

    def __str__(self):
        '''
        String repr of the test case
        '''
        return 'test_' + self.algorithm.__name__

    def set_pass_text(self, text):
        '''
        Setter for pass_text
        '''
        self.pass_text = text

    def set_fail_text(self, text):
        '''
        Setter for fail_text
        '''
        self.fail_text = text

    def set_skip_text(self, text):
        '''
        Setter for skip_text
        '''
        self.skip_text = text

    def generate_skip_text(self):
        '''
        Skip text is generated if there is no skip text (the case when test is skipped when the parent test cases fail or skip)
        To track down the root cause of this skip.
        '''
        text = str(self) + ' is skipped because '
        for test in self.parents:
            if test.result != 1:
                text = text + test.to_echo()
        return text

    def add_parent(self, parent_test_case):
        '''
        Adds a parent test case
        '''
        self.parents.append(parent_test_case)

    def add_child(self, child_test_case):
        '''
        Adds a child test case
        '''
        self.children.append(child_test_case)
        child_test_case.add_parent(self)

    def to_skip(self):
        '''
        Checks if any of the parent test cases failed or skipped which causes this case to skip
        '''
        for test in self.parents:
            if test.result != 1:
                print("{} - {}".format(str(test), str(test.result)), file=sys.stderr)
                return True
        return False

    def run(self, test_runner):
        '''
        First checks if the parent test cases were successful then run the text.
        '''
        # Checking if to skip
        if self.to_skip() is True:
            # warning will be generated because the test case is skipped because of some parent failure
            self.warning = True
            self.result = 0
            return
        # run the test if not skipped
        self.algorithm(self, test_runner)
        # if it fails it'll generate a warning
        if self.result == -1:
            self.warning = True

    def to_echo(self):
        '''
        Returns the text based on the result of the test case
        '''
        if self.result == 1:
            return self.pass_text
        elif self.result == -1:
            return self.fail_text
        elif self.result == 2:
            return 'Unknown error'
        elif self.skip_text == '':
            self.skip_text = self.generate_skip_text()
        return self.skip_text


def initiate_tests():
    '''
    Initiates test case objects and generates a test case graph for execution
    '''

    def base_algorithm(test, runner):
        if True is True:
            test.result = 1

    # Base test case
    test_base = Test(base_algorithm)

    # Info Success Test Cases

    test_info_implement = Test(info_implement)
    test_info_implement.set_pass_text('Info endpoint implemented by the server')
    test_info_implement.set_fail_text('Info endpoint not implemented by the server')

    test_info_implement_default = Test(info_implement_default)
    test_info_implement_default.set_pass_text('Info endpoint implemented with default encoding')
    test_info_implement_default.set_fail_text('Info endpoint not implemented with default encoding')

    test_info_circular = Test(info_circular)
    test_info_circular.set_pass_text('circular key in info response object')
    test_info_circular.set_fail_text('"circular" key not in info response object. It sends ')

    test_info_algorithms = Test(info_algorithms)
    test_info_algorithms.set_pass_text('algorithms key in info response object')
    test_info_algorithms.set_fail_text('"algorithms" key not in info response object. It sends ')

    test_info_subsequence_limit = Test(info_subsequence)
    test_info_subsequence_limit.set_pass_text('subsequence_limit key in info response object')
    test_info_subsequence_limit.set_fail_text('"subsequence_limit" key not in info response object. It sends ')

    test_info_api_version = Test(info_api_version)
    test_info_api_version.set_pass_text('supported_api_versions key in info response object')
    test_info_api_version.set_fail_text('supported_api_versions key not in info response object. It sends ')

    # Metadata Success Test Cases

    test_metadata_implement = Test(metadata_implement)
    test_metadata_implement.set_pass_text('Metadata endpoint implemented by the server')
    test_metadata_implement.set_fail_text('Metadata endpoint not implemented by the server')

    test_metadata_implement_default = Test(metadata_implement_default)
    test_metadata_implement_default.set_pass_text('Metadata endpoint implemented with default encoding')
    test_metadata_implement_default.set_fail_text('Metadata endpoint not implemented with default encoding')

    test_metadata_query_by_trunc512 = Test(metadata_query_by_trunc512)
    test_metadata_query_by_trunc512.set_pass_text('TRUNC512 algorithm is working in the server for metadata endpoint')
    test_metadata_query_by_trunc512.set_fail_text('TRUNC512 algorithm is not working in the server for metadata endpoint even though info endpoint indicates it"s support')

    test_metadata_query_circular_sequence = Test(metadata_query_circular_sequence)
    test_metadata_query_circular_sequence.set_pass_text('Circular sequence metadata can be retrieved')
    test_metadata_query_circular_sequence.set_fail_text('Circular sequence metadata can not be retrieved even though info endpoint indicates it"s support')

    test_metadata_md5 = Test(metadata_md5)
    test_metadata_md5.set_pass_text('md5 key in metadata response object')
    test_metadata_md5.set_fail_text('md5 key not in metadata response object instead sends ')

    test_metadata_trunc512 = Test(metadata_trunc512)
    test_metadata_trunc512.set_pass_text('trunc512 key in metadata response object')
    test_metadata_trunc512.set_fail_text('trunc512 key not in metadata response object even though info endpoint indicates it"s support instead sends ')

    test_metadata_length = Test(metadata_length)
    test_metadata_length.set_pass_text('length key in metadata response object')
    test_metadata_length.set_fail_text('length key not in metadata response object or incorrect value in "length" key instead sends ')

    test_metadata_aliases = Test(metadata_aliases)
    test_metadata_aliases.set_pass_text('aliases key in metadata response object')
    test_metadata_aliases.set_fail_text('aliases key not in metadata response object')

    test_metadata_invalid_checksum_404_error = Test(metadata_invalid_checksum_404_error)
    test_metadata_invalid_checksum_404_error.set_pass_text('Server is correctly sending 404 on invalid checksum')
    test_metadata_invalid_checksum_404_error.set_fail_text('Server is not sending 404 on invalid checksum instead sends ')

    test_metadata_invalid_encoding_406_error = Test(metadata_invalid_encoding_406_error)
    test_metadata_invalid_encoding_406_error.set_pass_text('Server is correctly sending 406 on invalid encoding')
    test_metadata_invalid_encoding_406_error.set_fail_text('Server is not sending 406 on invalid encoding instead sends ')

    # Sequence endpoint test cases

    test_sequence_implement = Test(sequence_implement)
    test_sequence_implement.set_pass_text('Sequence endpoint implemented in the server')
    test_sequence_implement.set_fail_text('Sequence endpoint not implemented in the server')

    test_sequence_implement_default = Test(sequence_implement_default)
    test_sequence_implement_default.set_pass_text('Sequence endpoint implemented with default encoding')
    test_sequence_implement_default.set_fail_text('Sequence endpoint not implemented with default encoding')

    test_sequence_query_by_trunc512 = Test(sequence_query_by_trunc512)
    test_sequence_query_by_trunc512.set_pass_text('TRUNC512 algorithm is working in the server for sequence endpoint')
    test_sequence_query_by_trunc512.set_fail_text('TRUNC512 algorithm is not working in the server for sequence endpoint even though info endpoint indicates it"s support')

    test_sequence_invalid_checksum_404_error = Test(sequence_invalid_checksum_404_error)
    test_sequence_invalid_checksum_404_error.set_pass_text('Server is correctly sending 404 on invalid checksum')
    test_sequence_invalid_checksum_404_error.set_fail_text('Server is not sending 404 on invalid checksum instead sends ')

    test_sequence_invalid_encoding_406_error = Test(sequence_invalid_encoding_406_error)
    test_sequence_invalid_encoding_406_error.set_pass_text('Server is correctly sending 406 on invalid encoding')
    test_sequence_invalid_encoding_406_error.set_fail_text('Server is not sending 406 on invalid encoding instead sends ')

    test_sequence_start_end = Test(sequence_start_end)
    test_sequence_start_end.set_pass_text('Server supports start end query params')
    test_sequence_start_end.set_fail_text('Server does not support start end query params')

    test_sequence_start_end_success_cases = Test(sequence_start_end_success_cases)
    test_sequence_start_end_success_cases.set_pass_text('Server passed all the success edge cases with start end query params')
    test_sequence_start_end_success_cases.set_fail_text('Server did not pass all the success edge cases with start end query params. Check <a href="https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/success/#sub-sequence-queries">https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/success/#sub-sequence-queries</a>')
    test_sequence_start_end_success_cases.cases = [
        (['?start=10&end=10', 10, 10], 0),
        (['?start=10&end=20', 10, 20], 10),
        (['?start=10&end=11', 10, 11], 1),
        (['?start=230208', 230208, None], 10),
        (['?end=5', None, 5], 5),
        (['?start=230217&end=230218', 230217, 230218], 1),
        (['?start=0', 0, None], 230218),
        (['?&end=230218', None, 230218], 230218),
        (['?start=0&end=230218', 0, 230218], 230218),
        (['?start=1&end=230218', 1, 230218], 230217),
        (['?start=230217', 230217, None], 1),
        (['?end=0', None, 0], 0)
    ]

    test_sequence_range = Test(sequence_range)
    test_sequence_range.set_pass_text('Server supports range header')
    test_sequence_range.set_fail_text('Server does not support range header')

    test_sequence_range_success_cases = Test(sequence_range_success_cases)
    test_sequence_range_success_cases.set_pass_text('Server passed all the success edge cases with range header query')
    test_sequence_range_success_cases.set_fail_text('Server did not pass all the success edge cases with range header query. Check <a href="https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/success/#sub-sequence-queries">https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/success/#sub-sequence-queries</a>')
    test_sequence_range_success_cases.cases = [
        (['bytes=10-19', 10, 19], [206, 10]),
        (['bytes=10-230217', 10, 230217], [206, 230208]),
        (['bytes=10-999999', 10, 999999], [206, 230208]),
        (['bytes=0-230217', 0, 230217], [200, 230218]),
        (['bytes=0-999999', 0, 999999], [200, 230218]),
        (['bytes=0-0', 0, 0], [206, 1]),
        (['bytes=230217-230217', 230217, 230217], [206, 1])
    ]

    test_sequence_circular = Test(sequence_circular)
    test_sequence_circular.set_pass_text('Circular sequence can be retrieved successfully passing all the edge cases')
    test_sequence_circular.set_fail_text('Circular sequences can not be retrieved even though info endpoint indicates its support')
    test_sequence_circular.cases = [
        ('?start=5374&end=5', ['ATCCAACCTGCAGAGTT', 17]),
        ('?start=5374&end=0', ['ATCCAACCTGCA', 12]),
        ('?start=5380&end=25', ['CCTGCAGAGTTTTATCGCTTCCATGACGCAG', 31]),
    ]

    test_sequence_start_end_errors = Test(sequence_start_end_errors)
    test_sequence_start_end_errors.set_pass_text('Server is correctly throwing errors for start-end error cases')
    test_sequence_start_end_errors.set_fail_text('Server is not correctly throwing errors for start-end error cases. Check <a href="https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-start-end-parameters">https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-start-end-parameters</a>')
    test_sequence_start_end_errors.cases = [

        (['6681ac2f62509cfc220d78751b8dc524', '?start=abc&end=20'], 400),
        (['6681ac2f62509cfc220d78751b8dc524', '?start=-10&end=-29', {}], 400),
        (['6681ac2f62509cfc220d78751b8dc524', '?start=abc'], 400),

        # Range out of bounds. Size of the sequence being tested is 5386.
        (['3332ed720ac7eaa9b3655c06f6b9e196', '?start=67&end=5387'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', '?start=5386&end=5375'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', '?start=5386&end=5386'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', '?start=5386&end=5'], 416),
    ]

    test_sequence_range_errors = Test(sequence_range_errors)
    test_sequence_range_errors.set_pass_text('Server is correctly throwing errors for range error cases')
    test_sequence_range_errors.set_fail_text('Server is not correctly throwing errors for range error cases. Check <a href="https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-range-header">https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-range-header</a>')
    test_sequence_range_errors.cases = [
        (['6681ac2f62509cfc220d78751b8dc524', 'units=20-30'], 400),
        (['6681ac2f62509cfc220d78751b8dc524', 'bytes=ab-19'], 400),
        (['6681ac2f62509cfc220d78751b8dc524', 'bytes=-10--19'], 400),
        (['6681ac2f62509cfc220d78751b8dc524', 'bytes=10--19'], 400),
        (['6681ac2f62509cfc220d78751b8dc524', 'bytes=-10-'], 400),
        (['6681ac2f62509cfc220d78751b8dc524', 'bytes==10-19'], 400),

        # Range out of bounds as fbs > lbs which is not allowed
        (['3332ed720ac7eaa9b3655c06f6b9e196', 'bytes=5200-19'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', 'bytes=59-50'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', 'bytes=5385-5382'], 416),

        # Range out of bounds. Size of the sequence tested is 5386
        (['3332ed720ac7eaa9b3655c06f6b9e196', 'bytes=5387-5391'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', 'bytes=5386-5387'], 416),
        (['3332ed720ac7eaa9b3655c06f6b9e196', 'bytes=9999-99999'], 416)
    ]

    test_sequence_circular_support_false_errors = Test(sequence_circular_support_false_errors)
    test_sequence_circular_support_false_errors.set_pass_text('Server is correctly throwing for error cases of circular sequences when server does not support circular sequences')
    test_sequence_circular_support_false_errors.set_fail_text('Server is not correctly throwing for error cases of circular sequences when server does not support circular sequences. Check <a href="https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-start-end-parameters">https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-start-end-parameters</a>')
    test_sequence_circular_support_false_errors.cases = [
        (['6681ac2f62509cfc220d78751b8dc524', '?start=220218&end=671'], 501),
        (['3332ed720ac7eaa9b3655c06f6b9e196', '?start=20&end=4'], 501)
    ]

    test_sequence_circular_support_true_errors = Test(sequence_circular_support_true_errors)
    test_sequence_circular_support_true_errors.set_pass_text('Server is correctly throwing for error cases of circular sequences when server supports circular sequences')
    test_sequence_circular_support_true_errors.set_fail_text('Server is not correctly throwing for error cases of circular sequences when server supports circular sequences. Check <a href="https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-start-end-parameters">https://compliancedoc.readthedocs.io/en/latest/GET_sequence_API/errors/#error-conditions-while-using-start-end-parameters</a>')
    test_sequence_circular_support_true_errors.cases = [
        (['6681ac2f62509cfc220d78751b8dc524', '?start=220218&end=671'], 416)
    ]

    # generating test graph

    test_base.add_child(test_info_implement)

    test_info_implement.add_child(test_info_implement_default)
    test_info_implement.add_child(test_info_circular)
    test_info_implement.add_child(test_info_algorithms)
    test_info_implement.add_child(test_info_subsequence_limit)
    test_info_implement.add_child(test_info_api_version)

    test_base.add_child(test_metadata_implement)

    test_metadata_implement.add_child(test_metadata_implement_default)

    test_metadata_implement.add_child(test_metadata_query_by_trunc512)
    test_info_algorithms.add_child(test_metadata_query_by_trunc512)

    test_metadata_implement.add_child(test_metadata_query_circular_sequence)
    test_info_circular.add_child(test_metadata_query_circular_sequence)

    test_metadata_implement.add_child(test_metadata_md5)

    test_metadata_implement.add_child(test_metadata_trunc512)
    test_info_algorithms.add_child(test_metadata_trunc512)

    test_metadata_implement.add_child(test_metadata_length)
    test_metadata_implement.add_child(test_metadata_aliases)
    test_metadata_implement.add_child(test_metadata_invalid_checksum_404_error)
    test_metadata_implement.add_child(test_metadata_invalid_encoding_406_error)

    test_base.add_child(test_sequence_implement)

    test_sequence_implement.add_child(test_sequence_implement_default)
    test_sequence_implement.add_child(test_sequence_start_end)
    test_sequence_implement.add_child(test_sequence_range)

    test_sequence_implement.add_child(test_sequence_query_by_trunc512)
    test_info_algorithms.add_child(test_sequence_query_by_trunc512)

    test_sequence_implement.add_child(test_sequence_invalid_checksum_404_error)

    test_sequence_implement.add_child(test_sequence_invalid_encoding_406_error)

    test_sequence_start_end.add_child(test_sequence_start_end_success_cases)
    test_sequence_range.add_child(test_sequence_range_success_cases)

    test_sequence_implement.add_child(test_sequence_circular)
    test_info_circular.add_child(test_sequence_circular)

    test_sequence_start_end.add_child(test_sequence_start_end_errors)

    test_sequence_range.add_child(test_sequence_range_errors)

    test_sequence_implement.add_child(test_sequence_circular_support_false_errors)
    test_info_circular.add_child(test_sequence_circular_support_false_errors)

    test_sequence_implement.add_child(test_sequence_circular_support_true_errors)
    test_info_circular.add_child(test_sequence_circular_support_true_errors)

    return test_base



import click
import os
import json
import tarfile
import sys

from compliance_suite.test_runner import TestRunner
from compliance_suite.report_server import start_mock_server

# try:
#     from compliance_suite.test_runner import TestRunner
#     from compliance_suite.report_server import start_mock_server
# except:
#     from test_runner import TestRunner
#     from report_server import start_mock_server


def valid_file_name(file_name, val):
    if os.path.exists(file_name):
        print('yo', file=sys.stderr)
        new_file_name = file_name + '_' + str(val)
        valid_file_name(new_file_name, val+1)
    return file_name

def scan_for_errors(json):
    '''
    Routine used to loop through the available results data structure and generate
    high-level summaries for the four main test routines

    - service info
    - metadata
    - sequence
    - sequence range
    '''
    high_level_summary={}
    for high_level_name in ('test_info_implement', 'test_metadata_implement', 'test_sequence_implement', 'test_sequence_range'):
        # We are successful unless proven otherwise
        result=1
        for test in json[0]["test_results"]:
            if high_level_name in test["parents"]:
                if test['warning']:
                    result = test["result"]
                    break
        high_level_summary[high_level_name] =  {
            'result' : result,
            'name': high_level_name
        }
    json[0]["high_level_summary"] = high_level_summary


@click.group()
def main():
    pass


@main.command(help='run compliance utility report using base urls')
@click.option('--server', '-s', multiple=True, help='base_url')
@click.option(
    '--file_path_name',
    '-fpn', default='web', help='to create a tar.gz file')
@click.option(
    '--json_path',
    '--json', help='create a json file report. Setting this to "-" will emit to standard out')
@click.option(
    '--serve', is_flag=True, help='spin up a server')
@click.option(
    '--no-web', is_flag=True, help='skip the creation of a tarball')
def report(server, file_path_name, json_path, serve, no_web):
    '''
    CLI command report to execute the report session and generate report on
    terminal, html file and json file if provided by the user

    Required arguments:
        server - At least one server is required. Multiple can be provided

    Optional arguments:
        --file_path_name - file name for w:gz file of web folder. Default is
        web_<int>.tar.gz
        --json_path - Provide a path to dump the final JSON content to
        --no-web - Avoid dumping a webfile
    '''
    final_json = []
    if len(server) == 0:
        raise Exception('No server url provided. Provide at least one')
    for s in server:
        tr = TestRunner(s)
        tr.run_tests()
        final_json.append(tr.generate_final_json())

    scan_for_errors(final_json)

    if json_path is not None:
        if json_path == '-':
            json.dump(final_json, sys.stdout)
        else:
            with open(json_path, 'w') as outfile:
                json.dump(final_json, outfile)

    WEB_DIR = os.path.join(os.path.dirname(__file__), 'web')

    if not no_web:
        if file_path_name is not None:
            with open(os.path.join(WEB_DIR, 'temp_result' + '.json'), 'w+') as outfile:
                json.dump(final_json, outfile)

            index = 0
            while(os.path.exists(file_path_name + '_' + str(index) + '.tar.gz')):
                index = index + 1
            with tarfile.open(file_path_name + '_' + str(index) + '.tar.gz', "w:gz") as tar:
                tar.add(WEB_DIR, arcname=os.path.basename(WEB_DIR))

    if serve is True:
        start_mock_server()


if __name__ == "__main__":
    main()



import http.server
import socketserver
import os
import socket
import webbrowser
import sys

WEB_DIR = os.path.join(os.path.dirname(__file__), 'web')


def get_free_port():
    ''' get_free_port function is used in conftest and the return of this
    is a free port available in the system on which the mock server will be
    run. This port will be passed to start_mock_server as a required parameter
    from conftest.py
    '''

    s = socket.socket(socket.AF_INET, type=socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    address, port = s.getsockname()
    s.close()
    return port


def start_mock_server(port=7878):
    os.chdir(WEB_DIR)
    port = get_free_port()
    Handler = http.server.SimpleHTTPRequestHandler
    httpd = socketserver.TCPServer(("", port), Handler)
    print("serving at http://localhost:" + str(port), file=sys.stderr)
    webbrowser.open("http://localhost:" + str(port))
    httpd.serve_forever()



import setuptools

# Work around mbcs bug in distutils.
# http://bugs.python.org/issue10945
import codecs
try:
    codecs.lookup('mbcs')
except LookupError:
    ascii = codecs.lookup('ascii')
    func = lambda name, enc=ascii: {True: enc}.get(name=='mbcs')
    codecs.register(func)

with open("README.md", "r") as fh:
    long_description = fh.read()
install_requires = ['requests', 'click']


setuptools.setup(
    name="refget-compliance",
    version="1.2.0",
    author="Somesh Chaturvedi",
    author_email="somesh.08.96@gmail.com",
    description="A compliance utility reporting system for refget server implementations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ga4gh/refget-compliance-suite",
    package_data={'': ['sequences/*', 'web/*']},
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    entry_points='''
        [console_scripts]
        refget-compliance=compliance_suite.cli:main
    ''',
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ),
)
